@@ -6,7 +6,7 @@
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
 
-#include "../../src/ttfautohint-1.5/frontend/maingui.h"
+#include "../../src/ttfautohint-1.6/frontend/maingui.h"
 #if !defined(Q_MOC_OUTPUT_REVISION)
 #error "The header file 'maingui.h' doesn't include <QObject>."
 #elif Q_MOC_OUTPUT_REVISION != 63
@@ -22,7 +22,7 @@ static const uint qt_meta_data_Main_GUI[] = {
        6,       // revision
        0,       // classname
        0,    0, // classinfo
-      21,   14, // methods
+      25,   14, // methods
        0,    0, // properties
        0,    0, // enums/sets
        0,    0, // constructors
@@ -35,22 +35,26 @@ static const uint qt_meta_data_Main_GUI[] = {
       33,    9,    9,    9, 0x08,
       49,    9,    9,    9, 0x08,
       66,    9,    9,    9, 0x08,
-      78,    9,    9,    9, 0x08,
-      90,    9,    9,    9, 0x08,
-     104,    9,    9,    9, 0x08,
-     119,    9,    9,    9, 0x08,
-     136,    9,    9,    9, 0x08,
-     156,    9,    9,    9, 0x08,
-     183,    9,    9,    9, 0x08,
-     200,    9,    9,    9, 0x08,
-     218,    9,    9,    9, 0x08,
-     237,    9,    9,    9, 0x08,
-     256,    9,    9,    9, 0x08,
-     278,    9,    9,    9, 0x08,
-     297,    9,    9,    9, 0x08,
-     311,    9,    9,    9, 0x08,
-     325,    9,    9,    9, 0x08,
-     337,    9,    9,    9, 0x08,
+      85,    9,    9,    9, 0x08,
+      97,    9,    9,    9, 0x08,
+     109,    9,    9,    9, 0x08,
+     123,    9,    9,    9, 0x08,
+     138,    9,    9,    9, 0x08,
+     155,    9,    9,    9, 0x08,
+     177,    9,    9,    9, 0x08,
+     204,    9,    9,    9, 0x08,
+     221,    9,    9,    9, 0x08,
+     239,    9,    9,    9, 0x08,
+     258,    9,    9,    9, 0x08,
+     279,    9,    9,    9, 0x08,
+     301,    9,    9,    9, 0x08,
+     320,    9,    9,    9, 0x08,
+     342,    9,    9,    9, 0x08,
+     361,    9,    9,    9, 0x08,
+     375,    9,    9,    9, 0x08,
+     389,    9,    9,    9, 0x08,
+     403,    9,    9,    9, 0x08,
+     415,    9,    9,    9, 0x08,
 
        0        // eod
 };
@@ -58,12 +62,14 @@ static const uint qt_meta_data_Main_GUI[] = {
 static const char qt_meta_stringdata_Main_GUI[] = {
     "Main_GUI\0\0about()\0browse_input()\0"
     "browse_output()\0browse_control()\0"
-    "check_min()\0check_max()\0check_limit()\0"
-    "check_dehint()\0check_no_limit()\0"
-    "check_no_increase()\0check_default_stem_width()\0"
-    "absolute_input()\0absolute_output()\0"
-    "absolute_control()\0check_number_set()\0"
-    "check_family_suffix()\0clear_status_bar()\0"
+    "browse_reference()\0check_min()\0"
+    "check_max()\0check_limit()\0check_dehint()\0"
+    "check_no_limit()\0check_no_x_increase()\0"
+    "check_default_stem_width()\0absolute_input()\0"
+    "absolute_output()\0absolute_control()\0"
+    "absolute_reference()\0set_ref_idx_box_max()\0"
+    "check_number_set()\0check_family_suffix()\0"
+    "clear_status_bar()\0start_timer()\0"
     "check_watch()\0watch_files()\0check_run()\0"
     "run()\0"
 };
@@ -78,23 +84,27 @@ void Main_GUI::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, vo
         case 1: _t->browse_input(); break;
         case 2: _t->browse_output(); break;
         case 3: _t->browse_control(); break;
-        case 4: _t->check_min(); break;
-        case 5: _t->check_max(); break;
-        case 6: _t->check_limit(); break;
-        case 7: _t->check_dehint(); break;
-        case 8: _t->check_no_limit(); break;
-        case 9: _t->check_no_increase(); break;
-        case 10: _t->check_default_stem_width(); break;
-        case 11: _t->absolute_input(); break;
-        case 12: _t->absolute_output(); break;
-        case 13: _t->absolute_control(); break;
-        case 14: _t->check_number_set(); break;
-        case 15: _t->check_family_suffix(); break;
-        case 16: _t->clear_status_bar(); break;
-        case 17: _t->check_watch(); break;
-        case 18: _t->watch_files(); break;
-        case 19: _t->check_run(); break;
-        case 20: _t->run(); break;
+        case 4: _t->browse_reference(); break;
+        case 5: _t->check_min(); break;
+        case 6: _t->check_max(); break;
+        case 7: _t->check_limit(); break;
+        case 8: _t->check_dehint(); break;
+        case 9: _t->check_no_limit(); break;
+        case 10: _t->check_no_x_increase(); break;
+        case 11: _t->check_default_stem_width(); break;
+        case 12: _t->absolute_input(); break;
+        case 13: _t->absolute_output(); break;
+        case 14: _t->absolute_control(); break;
+        case 15: _t->absolute_reference(); break;
+        case 16: _t->set_ref_idx_box_max(); break;
+        case 17: _t->check_number_set(); break;
+        case 18: _t->check_family_suffix(); break;
+        case 19: _t->clear_status_bar(); break;
+        case 20: _t->start_timer(); break;
+        case 21: _t->check_watch(); break;
+        case 22: _t->watch_files(); break;
+        case 23: _t->check_run(); break;
+        case 24: _t->run(); break;
         default: ;
         }
     }
@@ -133,9 +143,9 @@ int Main_GUI::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
     if (_id < 0)
         return _id;
     if (_c == QMetaObject::InvokeMetaMethod) {
-        if (_id < 21)
+        if (_id < 25)
             qt_static_metacall(this, _c, _id, _a);
-        _id -= 21;
+        _id -= 25;
     }
     return _id;
 }
