@@ -1,7 +1,7 @@
 /* tahints.c */
 
 /*
- * Copyright (C) 2011-2015 by Werner Lemberg.
+ * Copyright (C) 2011-2016 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -34,7 +34,15 @@ ta_axis_hints_new_segment(TA_AxisHints axis,
   TA_Segment segment = NULL;
 
 
-  if (axis->num_segments >= axis->max_segments)
+  if (axis->num_segments < TA_SEGMENTS_EMBEDDED)
+  {
+    if (axis->segments == NULL)
+    {
+      axis->segments = axis->embedded.segments;
+      axis->max_segments = TA_SEGMENTS_EMBEDDED;
+    }
+  }
+  else if (axis->num_segments >= axis->max_segments)
   {
     TA_Segment segments_new;
 
@@ -54,12 +62,26 @@ ta_axis_hints_new_segment(TA_AxisHints axis,
         || new_max > big_max)
       new_max = big_max;
 
-    segments_new = (TA_Segment)realloc(axis->segments,
-                                       new_max * sizeof (TA_SegmentRec));
-    if (!segments_new)
-      return FT_Err_Out_Of_Memory;
+    if (axis->segments == axis->embedded.segments)
+    {
+      axis->segments = (TA_Segment)malloc(
+                         (size_t)new_max * sizeof (TA_SegmentRec));
+      if (!axis->segments)
+        return FT_Err_Out_Of_Memory;
+
+      memcpy(axis->segments, axis->embedded.segments,
+             sizeof (axis->embedded.segments));
+    }
+    else
+    {
+      segments_new = (TA_Segment)realloc(
+                       axis->segments,
+                       (size_t)new_max * sizeof (TA_SegmentRec));
+      if (!segments_new)
+        return FT_Err_Out_Of_Memory;
+      axis->segments = segments_new;
+    }
 
-    axis->segments = segments_new;
     axis->max_segments = new_max;
   }
 
@@ -78,6 +100,7 @@ FT_Error
 ta_axis_hints_new_edge(TA_AxisHints axis,
                        FT_Int fpos,
                        TA_Direction dir,
+                       FT_Bool top_to_bottom_hinting,
                        TA_Edge* anedge)
 {
   FT_Error error = FT_Err_Ok;
@@ -85,7 +108,15 @@ ta_axis_hints_new_edge(TA_AxisHints axis,
   TA_Edge edges;
 
 
-  if (axis->num_edges >= axis->max_edges)
+  if (axis->num_edges < TA_EDGES_EMBEDDED)
+  {
+    if (axis->edges == NULL)
+    {
+      axis->edges = axis->embedded.edges;
+      axis->max_edges = TA_EDGES_EMBEDDED;
+    }
+  }
+  else if (axis->num_edges >= axis->max_edges)
   {
     TA_Edge edges_new;
 
@@ -105,12 +136,24 @@ ta_axis_hints_new_edge(TA_AxisHints axis,
         || new_max > big_max)
       new_max = big_max;
 
-    edges_new = (TA_Edge)realloc(axis->edges,
-                                 new_max * sizeof (TA_EdgeRec));
-    if (!edges_new)
-      return FT_Err_Out_Of_Memory;
+    if (axis->edges == axis->embedded.edges)
+    {
+      axis->edges = (TA_Edge)malloc((size_t)new_max * sizeof (TA_EdgeRec));
+      if (!axis->edges)
+        return FT_Err_Out_Of_Memory;
+
+      memcpy(axis->edges, axis->embedded.edges,
+             sizeof (axis->embedded.edges));
+    }
+    else
+    {
+      edges_new = (TA_Edge)realloc(axis->edges,
+                                   (size_t)new_max * sizeof (TA_EdgeRec));
+      if (!edges_new)
+        return FT_Err_Out_Of_Memory;
+      axis->edges = edges_new;
+    }
 
-    axis->edges = edges_new;
     axis->max_edges = new_max;
   }
 
@@ -119,7 +162,8 @@ ta_axis_hints_new_edge(TA_AxisHints axis,
 
   while (edge > edges)
   {
-    if (edge[-1].fpos < fpos)
+    if (top_to_bottom_hinting ? (edge[-1].fpos > fpos)
+                              : (edge[-1].fpos < fpos))
       break;
 
     /* we want the edge with same position and minor direction */
@@ -189,8 +233,84 @@ ta_dir_str(TA_Direction dir)
 
 
 #define TA_INDEX_NUM(ptr, base) \
-          ((ptr) ? ((ptr) - (base)) \
-                 : -1)
+          (int)((ptr) ? ((ptr) - (base)) \
+                      : -1)
+
+
+static char*
+ta_print_idx(char* p,
+             int idx)
+{
+  if (idx == -1)
+  {
+    p[0] = '-';
+    p[1] = '-';
+    p[2] = '\0';
+  }
+  else
+    sprintf(p, "%d", idx);
+
+  return p;
+}
+
+
+static int
+ta_get_segment_index(TA_GlyphHints hints,
+                     int point_idx,
+                     int dimension)
+{
+  TA_AxisHints axis = &hints->axis[dimension];
+  TA_Point point = hints->points + point_idx;
+  TA_Segment segments = axis->segments;
+  TA_Segment limit = segments + axis->num_segments;
+  TA_Segment segment;
+
+
+  for (segment = segments; segment < limit; segment++)
+  {
+    if (segment->first <= segment->last)
+    {
+      if (point >= segment->first && point <= segment->last)
+        break;
+    }
+    else
+    {
+      TA_Point p = segment->first;
+
+
+      for (;;)
+      {
+        if (point == p)
+          goto Exit;
+
+        if (p == segment->last)
+          break;
+
+        p = p->next;
+      }
+    }
+  }
+
+Exit:
+  if (segment == limit)
+    return -1;
+
+  return (int)(segment - segments);
+}
+
+
+static int
+ta_get_edge_index(TA_GlyphHints hints,
+                  int segment_idx,
+                  int dimension)
+{
+  TA_AxisHints axis = &hints->axis[dimension];
+  TA_Edge edges = axis->edges;
+  TA_Segment segment = axis->segments + segment_idx;
+
+
+  return segment_idx == -1 ? -1 : TA_INDEX_NUM(segment->edge, edges);
+}
 
 
 void
@@ -198,24 +318,50 @@ ta_glyph_hints_dump_points(TA_GlyphHints hints)
 {
   TA_Point points = hints->points;
   TA_Point limit = points + hints->num_points;
+  TA_Point* contour = hints->contours;
+  TA_Point* climit = contour + hints->num_contours;
   TA_Point point;
 
 
-  TA_LOG(("Table of points:\n"
-          "  [ index |  xorg |  yorg | xscale | yscale"
-          " |  xfit |  yfit | flags ]\n"));
+  TA_LOG(("Table of points:\n"));
+
+  if (hints->num_points)
+    TA_LOG(("  index  hedge  hseg  flags"
+            "  xorg  yorg  xscale  yscale   xfit    yfit"));
+  else
+    TA_LOG(("  (none)\n"));
 
   for (point = points; point < limit; point++)
-    TA_LOG(("  [ %5ld | %5d | %5d | %6.2f | %6.2f"
-            " | %5.2f | %5.2f |   %c   ]\n",
-            point - points,
+  {
+    int point_idx = TA_INDEX_NUM(point, points);
+    int segment_idx_1 = ta_get_segment_index(hints, point_idx, 1);
+
+    char buf1[16], buf2[16];
+
+
+    /* insert extra newline at the beginning of a contour */
+    if (contour < climit && *contour == point)
+    {
+      TA_LOG(("\n"));
+      contour++;
+    }
+
+    /* we don't show vertical edges since they are never used */
+    TA_LOG(("  %5d  %5s %5s  %s "
+            " %5d %5d %7.2f %7.2f %7.2f %7.2f\n",
+            point_idx,
+            ta_print_idx(buf1,
+                         ta_get_edge_index(hints, segment_idx_1, 1)),
+            ta_print_idx(buf2, segment_idx_1),
+            (point->flags & TA_FLAG_WEAK_INTERPOLATION) ? "weak" : " -- ",
+
             point->fx,
             point->fy,
             point->ox / 64.0,
             point->oy / 64.0,
             point->x / 64.0,
-            point->y / 64.0,
-            (point->flags & TA_FLAG_WEAK_INTERPOLATION) ? 'w' : ' '));
+            point->y / 64.0));
+  }
   TA_LOG(("\n"));
 }
 
@@ -267,30 +413,34 @@ ta_glyph_hints_dump_segments(TA_GlyphHints hints)
     TA_Segment limit = segments + axis->num_segments;
     TA_Segment seg;
 
+    char buf1[16], buf2[16], buf3[16];
+
 
     TA_LOG(("Table of %s segments:\n",
             dimension == TA_DIMENSION_HORZ ? "vertical"
                                            : "horizontal"));
     if (axis->num_segments)
-      TA_LOG(("  [ index |  pos  |  dir  | from"
-              " |  to  | link | serif | edge"
-              " | height | extra |    flags    ]\n"));
+      TA_LOG(("  index   pos   delta   dir   from   to"
+              "  link  serif  edge"
+              "  height  extra     flags\n"));
     else
       TA_LOG(("  (none)\n"));
 
     for (seg = segments; seg < limit; seg++)
-      TA_LOG(("  [ %5ld | %5.2g | %5s | %4ld"
-              " | %4ld | %4ld | %5ld | %4ld"
-              " | %6d | %5d | %11s ]\n",
-              seg - segments,
-              dimension == TA_DIMENSION_HORZ ? (int)seg->first->ox / 64.0
-                                             : (int)seg->first->oy / 64.0,
+      TA_LOG(("  %5d  %5d  %5d  %5s  %4d  %4d"
+              "  %4s  %5s  %4s"
+              "  %6d  %5d  %11s\n",
+              TA_INDEX_NUM(seg, segments),
+              seg->pos,
+              seg->delta,
               ta_dir_str((TA_Direction)seg->dir),
               TA_INDEX_NUM(seg->first, points),
               TA_INDEX_NUM(seg->last, points),
-              TA_INDEX_NUM(seg->link, segments),
-              TA_INDEX_NUM(seg->serif, segments),
-              TA_INDEX_NUM(seg->edge, edges),
+
+              ta_print_idx(buf1, TA_INDEX_NUM(seg->link, segments)),
+              ta_print_idx(buf2, TA_INDEX_NUM(seg->serif, segments)),
+              ta_print_idx(buf3, TA_INDEX_NUM(seg->edge, edges)),
+
               seg->height,
               seg->height - (seg->max_coord - seg->min_coord),
               ta_edge_flags_to_string(seg->flags)));
@@ -316,26 +466,37 @@ ta_glyph_hints_dump_edges(TA_GlyphHints hints)
     TA_Edge limit = edges + axis->num_edges;
     TA_Edge edge;
 
+    char buf1[16], buf2[16];
+
 
     /* note that TA_DIMENSION_HORZ corresponds to _vertical_ edges */
     /* since they have a constant X coordinate */
-    TA_LOG(("Table of %s edges:\n",
-            dimension == TA_DIMENSION_HORZ ? "vertical"
-                                           : "horizontal"));
+    if (dimension == TA_DIMENSION_HORZ)
+      TA_LOG(("Table of %s edges (1px=%.2fu, 10u=%.2fpx):\n",
+              "vertical",
+              65536.0 * 64.0 / hints->x_scale,
+              10.0 * hints->x_scale / 65536.0 / 64.0));
+    else
+      TA_LOG(("Table of %s edges (1px=%.2fu, 10u=%.2fpx):\n",
+              "horizontal",
+              65536.0 * 64.0 / hints->y_scale,
+              10.0 * hints->y_scale / 65536.0 / 64.0));
+
     if (axis->num_edges)
-      TA_LOG(("  [ index |  pos  |  dir  | link"
-              " | serif | blue | opos  |  pos  |    flags    ]\n"));
+      TA_LOG(("  index    pos     dir   link  serif"
+              "  blue    opos     pos       flags\n"));
     else
       TA_LOG(("  (none)\n"));
 
     for (edge = edges; edge < limit; edge++)
-      TA_LOG(("  [ %5ld | %5.2g | %5s | %4ld"
-              " | %5ld |   %c  | %5.2f | %5.2f | %11s ]\n",
-              edge - edges,
+      TA_LOG(("  %5d  %7.2f  %5s  %4s  %5s"
+              "    %c   %7.2f  %7.2f  %11s\n",
+              TA_INDEX_NUM(edge, edges),
               (int)edge->opos / 64.0,
               ta_dir_str((TA_Direction)edge->dir),
-              TA_INDEX_NUM(edge->link, edges),
-              TA_INDEX_NUM(edge->serif, edges),
+              ta_print_idx(buf1, TA_INDEX_NUM(edge->link, edges)),
+              ta_print_idx(buf2, TA_INDEX_NUM(edge->serif, edges)),
+
               edge->blue_edge ? 'y' : 'n',
               edge->opos / 64.0,
               edge->pos / 64.0,
@@ -383,15 +544,15 @@ ta_direction_compute(FT_Pos dx,
     else
     {
       dir = TA_DIR_DOWN;
-      ll = dy;
+      ll = -dy;
       ss = dx;
     }
   }
 
-  /* return no direction if arm lengths differ too much */
-  /* (value 14 is heuristic, corresponding to approx. 4.1 degrees) */
-  ss *= 14;
-  if (TA_ABS(ll) <= TA_ABS(ss))
+  /* return no direction if arm lengths do not differ enough */
+  /* (value 14 is heuristic, corresponding to approx. 4.1 degrees); */
+  /* the long arm is never negative */
+  if (ll <= 14 * TA_ABS(ss))
     dir = TA_DIR_NONE;
 
   return dir;
@@ -401,7 +562,8 @@ ta_direction_compute(FT_Pos dx,
 void
 ta_glyph_hints_init(TA_GlyphHints hints)
 {
-  memset(hints, 0, sizeof (TA_GlyphHintsRec));
+  /* no need to initialize the embedded items */
+  memset(hints, 0, sizeof (*hints) - sizeof (hints->embedded));
 }
 
 
@@ -423,24 +585,36 @@ ta_glyph_hints_done(TA_GlyphHints hints)
 
     axis->num_segments = 0;
     axis->max_segments = 0;
-    free(axis->segments);
-    axis->segments = NULL;
+    if (axis->segments != axis->embedded.segments)
+    {
+      free(axis->segments);
+      axis->segments = NULL;
+    }
 
     axis->num_edges = 0;
     axis->max_edges = 0;
-    free(axis->edges);
-    axis->edges = NULL;
+    if (axis->edges != axis->embedded.edges)
+    {
+      free(axis->edges);
+      axis->edges = NULL;
+    }
   }
 
-  free(hints->contours);
-  hints->contours = NULL;
+  if (hints->contours != hints->embedded.contours)
+  {
+    free(hints->contours);
+    hints->contours = NULL;
+  }
   hints->max_contours = 0;
   hints->num_contours = 0;
 
-  free(hints->points);
-  hints->points = NULL;
-  hints->num_points = 0;
+  if (hints->points != hints->embedded.points)
+  {
+    free(hints->points);
+    hints->points = NULL;
+  }
   hints->max_points = 0;
+  hints->num_points = 0;
 }
 
 
@@ -522,13 +696,25 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
   /* first of all, reallocate the contours array if necessary */
   new_max = (FT_UInt)outline->n_contours;
-  old_max = hints->max_contours;
-  if (new_max > old_max)
+  old_max = (FT_UInt)hints->max_contours;
+
+  if (new_max <= TA_CONTOURS_EMBEDDED)
+  {
+    if (hints->contours == NULL)
+    {
+      hints->contours = hints->embedded.contours;
+      hints->max_contours = TA_CONTOURS_EMBEDDED;
+    }
+  }
+  else if (new_max > old_max)
   {
     TA_Point* contours_new;
 
 
-    new_max = (new_max + 3) & ~3; /* round up to a multiple of 4 */
+    if (hints->contours == hints->embedded.contours)
+      hints->contours = NULL;
+
+    new_max = (new_max + 3) & ~3U; /* round up to a multiple of 4 */
 
     contours_new = (TA_Point*)realloc(hints->contours,
                                       new_max * sizeof (TA_Point));
@@ -536,19 +722,31 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
       return FT_Err_Out_Of_Memory;
 
     hints->contours = contours_new;
-    hints->max_contours = new_max;
+    hints->max_contours = (FT_Int)new_max;
   }
 
   /* reallocate the points arrays if necessary -- we reserve */
   /* two additional point positions, used to hint metrics appropriately */
   new_max = (FT_UInt)(outline->n_points + 2);
-  old_max = hints->max_points;
-  if (new_max > old_max)
+  old_max = (FT_UInt)hints->max_points;
+
+  if (new_max <= TA_POINTS_EMBEDDED)
+  {
+    if (hints->points == NULL)
+    {
+      hints->points = hints->embedded.points;
+      hints->max_points = TA_POINTS_EMBEDDED;
+    }
+  }
+  else if (new_max > old_max)
   {
     TA_Point points_new;
 
 
-    new_max = (new_max + 2 + 7) & ~7; /* round up to a multiple of 8 */
+    if (hints->points == hints->embedded.points)
+      hints->points = NULL;
+
+    new_max = (new_max + 2 + 7) & ~7U; /* round up to a multiple of 8 */
 
     points_new = (TA_Point)realloc(hints->points,
                                    new_max * sizeof (TA_PointRec));
@@ -556,7 +754,7 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
       return FT_Err_Out_Of_Memory;
 
     hints->points = points_new;
-    hints->max_points = new_max;
+    hints->max_points = (FT_Int)new_max;
   }
 
   hints->num_points = outline->n_points;
@@ -684,8 +882,6 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
         FT_Pos out_x, out_y;
 
-        FT_Bool is_first;
-
 
         /* since the first point of a contour could be part of a */
         /* series of near points, go backwards to find the first */
@@ -735,17 +931,13 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
         out_x = 0;
         out_y = 0;
 
-        is_first = 1;
-
-        for (point = first;
-             point != first || is_first;
-             point = point->next)
+        next = first;
+        do
         {
           TA_Direction out_dir;
 
 
-          is_first = 0;
-
+          point = next;
           next = point->next;
 
           out_x += next->fx - point->fx;
@@ -777,7 +969,8 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
           out_x = 0;
           out_y = 0;
-        }
+
+        } while (next != first);
       }
 
       /*
@@ -1249,33 +1442,27 @@ ta_iup_interp(TA_Point p1,
               TA_Point ref2)
 {
   TA_Point p;
-  FT_Pos u;
-  FT_Pos v1 = ref1->v;
-  FT_Pos v2 = ref2->v;
-  FT_Pos d1 = ref1->u - v1;
-  FT_Pos d2 = ref2->u - v2;
+  FT_Pos u, v1, v2, u1, u2, d1, d2;
 
 
   if (p1 > p2)
     return;
 
-  if (v1 == v2)
+  if (ref1->v > ref2->v)
   {
-    for (p = p1; p <= p2; p++)
-    {
-      u = p->v;
-
-      if (u <= v1)
-        u += d1;
-      else
-        u += d2;
-
-      p->u = u;
-    }
-    return;
+    p = ref1;
+    ref1 = ref2;
+    ref2 = p;
   }
 
-  if (v1 < v2)
+  v1 = ref1->v;
+  v2 = ref2->v;
+  u1 = ref1->u;
+  u2 = ref2->u;
+  d1 = u1 - v1;
+  d2 = u2 - v2;
+
+  if (u1 == u2 || v1 == v2)
   {
     for (p = p1; p <= p2; p++)
     {
@@ -1286,23 +1473,26 @@ ta_iup_interp(TA_Point p1,
       else if (u >= v2)
         u += d2;
       else
-        u = ref1->u + FT_MulDiv(u - v1, ref2->u - ref1->u, v2 - v1);
+        u = u1;
 
       p->u = u;
     }
   }
   else
   {
+    FT_Fixed scale = FT_DivFix(u2 - u1, v2 - v1);
+
+
     for (p = p1; p <= p2; p++)
     {
       u = p->v;
 
-      if (u <= v2)
-        u += d2;
-      else if (u >= v1)
+      if (u <= v1)
         u += d1;
+      else if (u >= v2)
+        u += d2;
       else
-        u = ref1->u + FT_MulDiv(u - v1, ref2->u - ref1->u, v2 - v1);
+        u = u1 + FT_MulFix(u - v1, scale);
 
       p->u = u;
     }
