@@ -1,7 +1,7 @@
 /* taprep.c */
 
 /*
- * Copyright (C) 2011-2015 by Werner Lemberg.
+ * Copyright (C) 2011-2016 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -19,7 +19,7 @@
 #define PREP(snippet_name) prep_ ## snippet_name
 
 
-const unsigned char PREP(hinting_limit_a) [] =
+static const unsigned char PREP(hinting_limit_a) [] =
 {
 
   /* all our measurements are taken along the y axis, */
@@ -34,7 +34,7 @@ const unsigned char PREP(hinting_limit_a) [] =
 
 /*  %d, hinting size limit */
 
-const unsigned char PREP(hinting_limit_b) [] =
+static const unsigned char PREP(hinting_limit_b) [] =
 {
 
   GT,
@@ -50,7 +50,7 @@ const unsigned char PREP(hinting_limit_b) [] =
 /* we store 0x10000 in CVT index `cvtl_funits_to_pixels' as a scaled value */
 /* to have a conversion factor from FUnits to pixels */
 
-const unsigned char PREP(store_funits_to_pixels) [] =
+static const unsigned char PREP(store_funits_to_pixels) [] =
 {
 
   PUSHB_1,
@@ -69,7 +69,7 @@ const unsigned char PREP(store_funits_to_pixels) [] =
 
 /* if the current ppem value is an exception, don't apply scaling */
 
-const unsigned char PREP(test_exception_a) [] =
+static const unsigned char PREP(test_exception_a) [] =
 {
 
   PUSHB_1,
@@ -82,7 +82,7 @@ const unsigned char PREP(test_exception_a) [] =
 
 /* provide scaling factors for all styles */
 
-const unsigned char PREP(align_top_a) [] =
+static const unsigned char PREP(align_x_height_a) [] =
 {
 
     PUSHB_2,
@@ -98,15 +98,15 @@ const unsigned char PREP(align_top_a) [] =
 /*    %c, style 0's x height blue zone idx */
 /*    %c, num_used_styles */
 
-const unsigned char PREP(align_top_b) [] =
+static const unsigned char PREP(align_x_height_b) [] =
 {
 
-      bci_align_top,
+      bci_align_x_height,
     LOOPCALL,
 
 };
 
-const unsigned char PREP(loop_cvt_a) [] =
+static const unsigned char PREP(loop_cvt_a) [] =
 {
 
     /* loop over (almost all) vertical CVT entries of all styles, part 1 */
@@ -127,7 +127,7 @@ const unsigned char PREP(loop_cvt_a) [] =
 /*        (std. width, widths, flat blues zones without artifical ones) */
 /*    %c, num_used_styles */
 
-const unsigned char PREP(loop_cvt_b) [] =
+static const unsigned char PREP(loop_cvt_b) [] =
 {
 
       bci_cvt_rescale_range,
@@ -149,7 +149,7 @@ const unsigned char PREP(loop_cvt_b) [] =
 /*    %c, style 0's number of round blue zones (without artificial ones) */
 /*    %c, num_used_styles */
 
-const unsigned char PREP(loop_cvt_c) [] =
+static const unsigned char PREP(loop_cvt_c) [] =
 {
 
       bci_cvt_rescale_range,
@@ -157,14 +157,14 @@ const unsigned char PREP(loop_cvt_c) [] =
 
 };
 
-const unsigned char PREP(test_exception_b) [] =
+static const unsigned char PREP(test_exception_b) [] =
 {
 
   EIF,
 
 };
 
-const unsigned char PREP(store_vwidth_data_a) [] =
+static const unsigned char PREP(store_vwidth_data_a) [] =
 {
 
   PUSHB_2,
@@ -174,7 +174,7 @@ const unsigned char PREP(store_vwidth_data_a) [] =
 
 /*  %c, offset to vertical width offset data in CVT */
 
-const unsigned char PREP(store_vwidth_data_b) [] =
+static const unsigned char PREP(store_vwidth_data_b) [] =
 {
 
   WS,
@@ -187,7 +187,7 @@ const unsigned char PREP(store_vwidth_data_b) [] =
 /*  %d, style 0's first vertical width index (in multiples of 64) */
 /*  %d, num_used_styles */
 
-const unsigned char PREP(store_vwidth_data_c) [] =
+static const unsigned char PREP(store_vwidth_data_c) [] =
 {
 
     0x00, /* high byte */
@@ -201,7 +201,7 @@ const unsigned char PREP(store_vwidth_data_c) [] =
 
 /*  %c, offset to vertical width size data in CVT */
 
-const unsigned char PREP(store_vwidth_data_d) [] =
+static const unsigned char PREP(store_vwidth_data_d) [] =
 {
 
   WS,
@@ -214,7 +214,7 @@ const unsigned char PREP(store_vwidth_data_d) [] =
 /*  %d, style 0's number of vertical widths (in multiples of 64) */
 /*  %d, num_used_styles */
 
-const unsigned char PREP(store_vwidth_data_e) [] =
+static const unsigned char PREP(store_vwidth_data_e) [] =
 {
 
     0x00, /* high byte */
@@ -223,7 +223,7 @@ const unsigned char PREP(store_vwidth_data_e) [] =
 
 };
 
-const unsigned char PREP(set_smooth_or_strong_a) [] =
+static const unsigned char PREP(set_smooth_or_strong_a) [] =
 {
 
   /*
@@ -251,7 +251,7 @@ const unsigned char PREP(set_smooth_or_strong_a) [] =
 
 /*  %c, either 0 or 100 */
 
-const unsigned char PREP(set_smooth_or_strong_b) [] =
+static const unsigned char PREP(set_smooth_or_strong_b) [] =
 {
 
   WCVTP,
@@ -277,7 +277,7 @@ const unsigned char PREP(set_smooth_or_strong_b) [] =
 
 /*      %c, either 0 or 100 */
 
-const unsigned char PREP(set_smooth_or_strong_c) [] =
+static const unsigned char PREP(set_smooth_or_strong_c) [] =
 {
 
       WCVTP,
@@ -314,7 +314,7 @@ const unsigned char PREP(set_smooth_or_strong_c) [] =
 
 /*          %c, either 0 or 100 */
 
-const unsigned char PREP(set_smooth_or_strong_d) [] =
+static const unsigned char PREP(set_smooth_or_strong_d) [] =
 {
 
           WCVTP,
@@ -333,7 +333,7 @@ const unsigned char PREP(set_smooth_or_strong_d) [] =
 /*  %c, style 0's number of blue ref indices */
 /*  %c, num_used_styles */
 
-const unsigned char PREP(round_blues) [] =
+static const unsigned char PREP(round_blues) [] =
 {
 
     bci_blue_round_range,
@@ -341,7 +341,7 @@ const unsigned char PREP(round_blues) [] =
 
 };
 
-const unsigned char PREP(set_dropout_mode) [] =
+static const unsigned char PREP(set_dropout_mode) [] =
 {
 
   PUSHW_1,
@@ -354,7 +354,7 @@ const unsigned char PREP(set_dropout_mode) [] =
 
 };
 
-const unsigned char PREP(reset_component_counter) [] =
+static const unsigned char PREP(reset_component_counter) [] =
 {
 
   /* In case an application tries to render `.ttfautohint' */
@@ -376,7 +376,7 @@ const unsigned char PREP(reset_component_counter) [] =
 
 };
 
-const unsigned char PREP(adjust_delta_exceptions) [] =
+static const unsigned char PREP(adjust_delta_exceptions) [] =
 {
 
   /* set delta base */
@@ -387,20 +387,29 @@ const unsigned char PREP(adjust_delta_exceptions) [] =
 };
 
 
-const unsigned char PREP(do_iup_y) [] =
+static const unsigned char PREP(set_default_cvs_values) [] =
 {
 
   /* We set a default value for `cvtl_do_iup_y'. */
   /* If we have delta exceptions before IUP_y, */
   /* the glyph's bytecode sets this CVT value temporarily to zero */
   /* and manually inserts IUP_y afterwards. */
-  /* It would be more elegant to use a storage area location instead, */
-  /* however, it is not possible to have a default value for them */
+
+  /* We set a default value for `cvtl_ignore_std_width'. */
+  /* As the name implies, the stem width computation routines */
+  /* ignore the standard width(s) if this flag gets set. */
+
+  /* It would be more elegant to use storage area locations instead, */
+  /* however, it is not possible to have default values for them */
   /* since storage area locations might be reset on a per-glyph basis */
   /* (this is dependent on the bytecode interpreter implementation). */
-  PUSHB_2,
+
+  PUSHB_4,
     cvtl_do_iup_y,
     100,
+    cvtl_ignore_std_width,
+    0,
+  WCVTP,
   WCVTP,
 
 };
@@ -492,24 +501,28 @@ TA_sfnt_build_number_set(SFNT* sfnt,
   nr = number_set;
   while (nr)
   {
-    if (nr->start == nr->end)
+    FT_UInt start = (FT_UInt)nr->start;
+    FT_UInt end = (FT_UInt)nr->end;
+
+
+    if (start == end)
     {
-      if (nr->start < 256)
-        *(single_arg--) = nr->start;
+      if (start < 256)
+        *(single_arg--) = start;
       else
-        *(single2_arg--) = nr->start;
+        *(single2_arg--) = start;
     }
     else
     {
-      if (nr->start < 256 && nr->end < 256)
+      if (start < 256 && end < 256)
       {
-        *(range_arg--) = nr->start;
-        *(range_arg--) = nr->end;
+        *(range_arg--) = start;
+        *(range_arg--) = end;
       }
       else
       {
-        *(range2_arg--) = nr->start;
-        *(range2_arg--) = nr->end;
+        *(range2_arg--) = start;
+        *(range2_arg--) = end;
       }
     }
     nr = nr->next;
@@ -540,9 +553,9 @@ TA_sfnt_build_number_set(SFNT* sfnt,
   if (have_range)
     BCI(CALL);
 
-  num_stack_elements = num_singles + num_singles2;
+  num_stack_elements = (FT_UShort)(num_singles + num_singles2);
   if (num_stack_elements > num_ranges + num_ranges2)
-    num_stack_elements = num_ranges + num_ranges2;
+    num_stack_elements = (FT_UShort)(num_ranges + num_ranges2);
   num_stack_elements += ADDITIONAL_STACK_ELEMENTS;
   if (num_stack_elements > sfnt->max_stack_elements)
     sfnt->max_stack_elements = num_stack_elements;
@@ -573,6 +586,8 @@ TA_table_build_prep(FT_Byte** prep,
 {
   SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
   glyf_Data* data = (glyf_Data*)glyf_table->data;
+  /* XXX: make this work for more than 256 styles */
+  FT_Byte num_used_styles = (FT_Byte)data->num_used_styles;
 
   FT_Int i;
 
@@ -593,7 +608,7 @@ TA_table_build_prep(FT_Byte** prep,
       return FT_Err_Out_Of_Memory;
   }
 
-  buf_len = bufp - buf;
+  buf_len = (FT_UInt)(bufp - buf);
   buf_new_len = buf_len;
 
   if (font->hinting_limit)
@@ -606,19 +621,16 @@ TA_table_build_prep(FT_Byte** prep,
   if (font->x_height_snapping_exceptions)
     buf_new_len += sizeof (PREP(test_exception_a));
 
-  buf_new_len += sizeof (PREP(align_top_a))
-                 + (data->num_used_styles > 6
-                      ? data->num_used_styles + 3
-                      : data->num_used_styles + 2)
-                 + sizeof (PREP(align_top_b));
+  buf_new_len += sizeof (PREP(align_x_height_a))
+                 + (num_used_styles > 6 ? num_used_styles + 3
+                                        : num_used_styles + 2)
+                 + sizeof (PREP(align_x_height_b));
   buf_new_len += sizeof (PREP(loop_cvt_a))
-                 + (data->num_used_styles > 3
-                     ? 2 * data->num_used_styles + 3
-                     : 2 * data->num_used_styles + 2)
+                 + (num_used_styles > 3 ? 2 * num_used_styles + 3
+                                        : 2 * num_used_styles + 2)
                  + sizeof (PREP(loop_cvt_b))
-                 + (data->num_used_styles > 3
-                     ? 2 * data->num_used_styles + 3
-                     : 2 * data->num_used_styles + 2)
+                 + (num_used_styles > 3 ? 2 * num_used_styles + 3
+                                        : 2 * num_used_styles + 2)
                  + sizeof (PREP(loop_cvt_c));
 
   if (font->x_height_snapping_exceptions)
@@ -627,15 +639,13 @@ TA_table_build_prep(FT_Byte** prep,
   buf_new_len += sizeof (PREP(store_vwidth_data_a))
                  + 1
                  + sizeof (PREP(store_vwidth_data_b))
-                 + (data->num_used_styles > 6
-                      ? 2 * (data->num_used_styles + 1) + 2
-                      : 2 * (data->num_used_styles + 1) + 1)
+                 + (num_used_styles > 6 ? 2 * (num_used_styles + 1) + 2
+                                        : 2 * (num_used_styles + 1) + 1)
                  + sizeof (PREP(store_vwidth_data_c))
                  + 1
                  + sizeof (PREP(store_vwidth_data_d))
-                 + (data->num_used_styles > 6
-                      ? 2 * (data->num_used_styles + 1) + 2
-                      : 2 * (data->num_used_styles + 1) + 1)
+                 + (num_used_styles > 6 ? 2 * (num_used_styles + 1) + 2
+                                        : 2 * (num_used_styles + 1) + 1)
                  + sizeof (PREP(store_vwidth_data_e));
   buf_new_len += sizeof (PREP(set_smooth_or_strong_a))
                  + 1
@@ -644,18 +654,17 @@ TA_table_build_prep(FT_Byte** prep,
                  + sizeof (PREP(set_smooth_or_strong_c))
                  + 1
                  + sizeof (PREP(set_smooth_or_strong_d));
-  buf_new_len += (data->num_used_styles > 3
-                     ? 2 * data->num_used_styles + 3
-                     : 2 * data->num_used_styles + 2)
+  buf_new_len += (num_used_styles > 3 ? 2 * num_used_styles + 3
+                                      : 2 * num_used_styles + 2)
                  + sizeof (PREP(round_blues));
   buf_new_len += sizeof (PREP(set_dropout_mode));
   buf_new_len += sizeof (PREP(reset_component_counter));
   if (font->control_data_head)
     buf_new_len += sizeof (PREP(adjust_delta_exceptions));
-  buf_new_len += sizeof (PREP(do_iup_y));
+  buf_new_len += sizeof (PREP(set_default_cvs_values));
 
   /* buffer length must be a multiple of four */
-  len = (buf_new_len + 3) & ~3;
+  len = (buf_new_len + 3) & ~3U;
   buf_new = (FT_Byte*)realloc(buf, len);
   if (!buf_new)
   {
@@ -686,14 +695,14 @@ TA_table_build_prep(FT_Byte** prep,
   if (font->x_height_snapping_exceptions)
     COPY_PREP(test_exception_a);
 
-  COPY_PREP(align_top_a);
-  if (data->num_used_styles > 6)
+  COPY_PREP(align_x_height_a);
+  if (num_used_styles > 6)
   {
     BCI(NPUSHB);
-    BCI(data->num_used_styles + 2);
+    BCI(num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + data->num_used_styles + 2);
+    BCI(PUSHB_1 - 1 + num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
   for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
@@ -704,17 +713,17 @@ TA_table_build_prep(FT_Byte** prep,
                   ? 0
                   : (unsigned char)CVT_X_HEIGHT_BLUE_OFFSET(i);
   }
-  *(bufp++) = data->num_used_styles;
-  COPY_PREP(align_top_b);
+  *(bufp++) = num_used_styles;
+  COPY_PREP(align_x_height_b);
 
   COPY_PREP(loop_cvt_a);
-  if (data->num_used_styles > 3)
+  if (num_used_styles > 3)
   {
     BCI(NPUSHB);
-    BCI(2 * data->num_used_styles + 2);
+    BCI(2 * num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + 2 * data->num_used_styles + 2);
+    BCI(PUSHB_1 - 1 + 2 * num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
   for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
@@ -726,17 +735,20 @@ TA_table_build_prep(FT_Byte** prep,
     *(bufp++) = (unsigned char)(
                   1
                   + CVT_VERT_WIDTHS_SIZE(i)
-                  + (CVT_BLUES_SIZE(i) > 1 ? CVT_BLUES_SIZE(i) - 2 : 0));
+                  + (CVT_BLUES_SIZE(i) > 1
+                     ? CVT_BLUES_SIZE(i) - (font->windows_compatibility ? 2
+                                                                        : 0)
+                     : 0));
   }
-  *(bufp++) = data->num_used_styles;
+  *(bufp++) = num_used_styles;
   COPY_PREP(loop_cvt_b);
-  if (data->num_used_styles > 3)
+  if (num_used_styles > 3)
   {
     BCI(NPUSHB);
-    BCI(2 * data->num_used_styles + 2);
+    BCI(2 * num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + 2 * data->num_used_styles + 2);
+    BCI(PUSHB_1 - 1 + 2 * num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
   for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
@@ -746,9 +758,12 @@ TA_table_build_prep(FT_Byte** prep,
     /* don't loop over artificial blue zones */
     *(bufp++) = (unsigned char)CVT_BLUE_SHOOTS_OFFSET(i);
     *(bufp++) = (unsigned char)(
-                  CVT_BLUES_SIZE(i) > 1 ? CVT_BLUES_SIZE(i) - 2 : 0);
+                  CVT_BLUES_SIZE(i) > 1
+                  ? CVT_BLUES_SIZE(i) - (font->windows_compatibility ? 2
+                                                                     : 0)
+                  : 0);
   }
-  *(bufp++) = data->num_used_styles;
+  *(bufp++) = num_used_styles;
   COPY_PREP(loop_cvt_c);
 
   if (font->x_height_snapping_exceptions)
@@ -757,13 +772,13 @@ TA_table_build_prep(FT_Byte** prep,
   COPY_PREP(store_vwidth_data_a);
   *(bufp++) = (unsigned char)CVT_VWIDTH_OFFSET_DATA(0);
   COPY_PREP(store_vwidth_data_b);
-  if (data->num_used_styles > 6)
+  if (num_used_styles > 6)
   {
     BCI(NPUSHW);
-    BCI(data->num_used_styles + 2);
+    BCI(num_used_styles + 2);
   }
   else
-    BCI(PUSHW_1 - 1 + data->num_used_styles + 2);
+    BCI(PUSHW_1 - 1 + num_used_styles + 2);
   for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
     if (data->style_ids[i] == 0xFFFFU)
@@ -772,18 +787,18 @@ TA_table_build_prep(FT_Byte** prep,
     *(bufp++) = HIGH(CVT_VERT_WIDTHS_OFFSET(i) * 64);
     *(bufp++) = LOW(CVT_VERT_WIDTHS_OFFSET(i) * 64);
   }
-  *(bufp++) = HIGH(data->num_used_styles);
-  *(bufp++) = LOW(data->num_used_styles);
+  *(bufp++) = HIGH(num_used_styles);
+  *(bufp++) = LOW(num_used_styles);
   COPY_PREP(store_vwidth_data_c);
   *(bufp++) = (unsigned char)CVT_VWIDTH_SIZE_DATA(0);
   COPY_PREP(store_vwidth_data_d);
-  if (data->num_used_styles > 6)
+  if (num_used_styles > 6)
   {
     BCI(NPUSHW);
-    BCI(data->num_used_styles + 2);
+    BCI(num_used_styles + 2);
   }
   else
-    BCI(PUSHW_1 - 1 + data->num_used_styles + 2);
+    BCI(PUSHW_1 - 1 + num_used_styles + 2);
   for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
     if (data->style_ids[i] == 0xFFFFU)
@@ -792,8 +807,8 @@ TA_table_build_prep(FT_Byte** prep,
     *(bufp++) = HIGH(CVT_VERT_WIDTHS_SIZE(i) * 64);
     *(bufp++) = LOW(CVT_VERT_WIDTHS_SIZE(i) * 64);
   }
-  *(bufp++) = HIGH(data->num_used_styles);
-  *(bufp++) = LOW(data->num_used_styles);
+  *(bufp++) = HIGH(num_used_styles);
+  *(bufp++) = LOW(num_used_styles);
   COPY_PREP(store_vwidth_data_e);
 
   COPY_PREP(set_smooth_or_strong_a);
@@ -804,13 +819,13 @@ TA_table_build_prep(FT_Byte** prep,
   *(bufp++) = font->dw_cleartype_strong_stem_width ? 100 : 0;
   COPY_PREP(set_smooth_or_strong_d);
 
-  if (data->num_used_styles > 3)
+  if (num_used_styles > 3)
   {
     BCI(NPUSHB);
-    BCI(2 * data->num_used_styles + 2);
+    BCI(2 * num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + 2 * data->num_used_styles + 2);
+    BCI(PUSHB_1 - 1 + 2 * num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
   for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
@@ -820,14 +835,14 @@ TA_table_build_prep(FT_Byte** prep,
     *(bufp++) = (unsigned char)CVT_BLUE_REFS_OFFSET(i);
     *(bufp++) = (unsigned char)CVT_BLUES_SIZE(i);
   }
-  *(bufp++) = data->num_used_styles;
+  *(bufp++) = num_used_styles;
   COPY_PREP(round_blues);
 
   COPY_PREP(set_dropout_mode);
   COPY_PREP(reset_component_counter);
   if (font->control_data_head)
     COPY_PREP(adjust_delta_exceptions);
-  COPY_PREP(do_iup_y);
+  COPY_PREP(set_default_cvs_values);
 
   *prep = buf;
   *prep_len = buf_new_len;
